from django.db.models.signals import post_save, post_delete
from django.dispatch import receiver
from django.conf import settings
from .backends import FireStore, MockNoSQL
from enum import Enum

class BACKENDS(Enum):
    FIRESTORE = 'firestore'
    MOCK = 'mock'

class SYNC_TYPE(Enum):
    UPDATE = 'update'
    DELETE = 'delete'

def get_readonly_backend(backend):
    if backend == BACKENDS.FIRESTORE.value:
        return FireStore(settings.FIRESTORE_CREDENTIALS_FILE)
    if backend == BACKENDS.MOCK.value:
        return MockNoSQL()

# todo: run these async
def perform_sync(instance, sync_type=SYNC_TYPE.UPDATE, created = False):
    model_requires_sync = getattr(instance, 'readonly_sync', False) == True

    if model_requires_sync:
        backends = getattr(settings, 'NOSQL_BACKENDS', [])
        for backend_name in backends:
            backend = get_readonly_backend(backend_name)
            if backend is not None:
                if sync_type == SYNC_TYPE.DELETE:
                    backend.delete(instance)
                else:
                    backend.sync(instance, created)


@receiver(post_save, dispatch_uid="django_nosql.sync")
def sync_readonly_db(sender, instance, created, **kwargs):
    perform_sync(instance, SYNC_TYPE.UPDATE, created)

@receiver(post_delete, dispatch_uid="django_nosql.sync.delete")
def sync_remove_readonly_db(sender, instance, **kwargs):
    perform_sync(instance, SYNC_TYPE.DELETE)



import importlib, firebase_admin
from firebase_admin import credentials, firestore


class NoSQLBackend:

    def __get_serializer_class(self, serializer_path):
        bits = serializer_path.split('.')
        class_name = bits.pop()
        module_string = (".").join(bits)
        mod = importlib.import_module(module_string)
        return getattr(mod, class_name)

    def get_serialized(self, instance, created = False):
        serializer = self.__get_serializer_class(instance.serializer_path)
        dta = serializer(instance).data
        id = str(instance.id)
        return (id, dta)


class MockNoSQL(NoSQLBackend):

    def sync(self, instance, created = False):
        id, dta = self.get_serialized(instance, created)
        return dta

    def delete(self, instance):
        return instance


class FireStore(NoSQLBackend):

    def __init__(self, path_to_credentials):
        super().__init__()
        try:
            firebase_admin.get_app()
        except ValueError:
            cred = credentials.Certificate(path_to_credentials)
            app = firebase_admin.initialize_app(cred)

        self.db = firestore.client()

    def sync(self, instance, created = False):
        id, dta = self.get_serialized(instance, created)
        return self.db.collection(instance.collection).document(id).set(dta)

    def delete(self, instance):
        return self.db.collection(instance.collection)\
            .document(str(instance.id))\
            .delete()



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.test import TestCase, override_settings
from rest_framework import routers, serializers, viewsets
from django.contrib.auth import get_user_model
from django.conf import settings

def create_fake_user():
    return get_user_model().objects.create(username='fake_user', password='testtest')

from .signals import FireStore

# Serializers define the API representation.
class UserSerializer(serializers.ModelSerializer):
    class Meta:
        model = get_user_model()
        fields = '__all__'

fb = FireStore(settings.FIRESTORE_CREDENTIALS_FILE)

class SignalsTestCase(TestCase):

    def setUp(self):
        self.user = create_fake_user()
        self.user.collection = 'test_users'
        self.user.serializer_path = 'django_nosql.tests.UserSerializer'

    def test_it_syncs_wth_firestore(self):
        fb.sync(self.user)

    def test_it_deletes_from_firestore(self):
        fb.delete(self.user)


import setuptools

with open("./README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="django_nosql",
    version="0.0.2",
    author="Christo Crampton",
    author_email="christo@appointmentguru.co",
    description="Stream model updates to nosql backends",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://gitlab.com/SchoolOrchestration/django_nosql.git",
    packages=['django_nosql'],
    classifiers=[
        "Programming Language :: Python :: 3",
        "Operating System :: OS Independent",
    ],
)
